package partisan.weforge.xyz.pulse

import android.Manifest
import android.app.role.RoleManager
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.provider.Settings
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.core.widget.doAfterTextChanged
import java.lang.NumberFormatException
import android.text.InputType
import android.widget.CheckBox
import android.widget.EditText
import android.widget.LinearLayout
import android.view.ViewGroup.LayoutParams.WRAP_CONTENT
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.ItemTouchHelper
import partisan.weforge.xyz.pulse.getServicePriority
import partisan.weforge.xyz.pulse.setServicePriority
import partisan.weforge.xyz.pulse.isServiceEnabled
import partisan.weforge.xyz.pulse.setServiceEnabled
import partisan.weforge.xyz.pulse.databinding.ActivityMainBinding

class MainActivity : AppCompatActivity() {
    companion object {
        private val PERMISSIONS = arrayOf(
            Manifest.permission.READ_CONTACTS,
            Manifest.permission.CALL_PHONE,
        )
    }

    private lateinit var binding: ActivityMainBinding
    private lateinit var prefs: Preferences
    private lateinit var window: PopupWindow
    private var roleManager: RoleManager? = null

    private val registerForCallRedirectionRole =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {}

    private val registerForGeneralPermissions =
        registerForActivityResult(ActivityResultContracts.RequestMultiplePermissions()) {}

    private val registerForDrawOverlays =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {}

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)
        init()
        setup()
    }

    override fun onDestroy() {
        super.onDestroy()
        window.cancel()
    }

    private fun init() {
        prefs = Preferences(this)
        window = PopupWindow(this, null)
        roleManager = getSystemService(RoleManager::class.java)
        binding.apply {
            redirectionDelay.value = (prefs.redirectionDelay / 1000).toFloat()
            popupPosition.editText?.setText(prefs.popupPosition.toString())
            toggle.isChecked = prefs.isEnabled
        }
    }

    private fun setup() {
        binding.apply {
            redirectionDelay.setLabelFormatter {
                String.format("%.1f", it)
            }
            redirectionDelay.addOnChangeListener { _, value, _ ->
                prefs.redirectionDelay = (value * 1000).toLong()
            }
            popupPosition.setEndIconOnClickListener {
                window.preview()
            }
            popupPosition.editText?.doAfterTextChanged {
                try {
                    prefs.popupPosition = it?.toString()?.toInt() ?: return@doAfterTextChanged
                } catch (exc: NumberFormatException) {}
            }
            toggle.setOnCheckedChangeListener { _, isChecked ->
                if (isChecked && !hasPermissions()) {
                    toggle.isChecked = false
                    requestPermissions()
                    return@setOnCheckedChangeListener
                }
                prefs.isEnabled = isChecked
            }
            val services = listOf(
                ServiceEntry("vnd.android.cursor.item/vnd.org.thoughtcrime.securesms.call", R.string.destination_signal, this@MainActivity.isServiceEnabled("vnd.android.cursor.item/vnd.org.thoughtcrime.securesms.call")),
                ServiceEntry("vnd.android.cursor.item/vnd.org.telegram.messenger.android.call", R.string.destination_telegram, this@MainActivity.isServiceEnabled("vnd.android.cursor.item/vnd.org.telegram.messenger.android.call")),
                ServiceEntry("vnd.android.cursor.item/vnd.ch.threema.app.call", R.string.destination_threema, this@MainActivity.isServiceEnabled("vnd.android.cursor.item/vnd.ch.threema.app.call")),
                ServiceEntry("vnd.android.cursor.item/vnd.com.whatsapp.voip.call", R.string.destination_whatsapp, this@MainActivity.isServiceEnabled("vnd.android.cursor.item/vnd.com.whatsapp.voip.call")),
            )

            val adapter = ServiceAdapter(
                context = this@MainActivity,
                services = services.toMutableList(),
                onReordered = { updatedList ->
                    updatedList.forEachIndexed { index, entry ->
                        setServicePriority(entry.mimetype, index)
                    }
                }
            )
            binding.serviceRecycler.adapter = adapter
            binding.serviceRecycler.layoutManager = LinearLayoutManager(this@MainActivity)

            val touchHelper = ItemTouchHelper(adapter.dragHelper)
            touchHelper.attachToRecyclerView(binding.serviceRecycler)

            adapter.setDragStartListener { viewHolder ->
                touchHelper.startDrag(viewHolder)
            }

            // binding.serviceConfigList.removeAllViews()
            // for ((mimetype, labelRes) in mimetypes) {
            //     val checkbox = CheckBox(this@MainActivity).apply {
            //         text = getString(labelRes)
            //         isChecked = this@MainActivity.isServiceEnabled(mimetype)
            //         setOnCheckedChangeListener { _, checked ->
            //             this@MainActivity.setServiceEnabled(mimetype, checked)
            //         }
            //     }
            
            //     val priorityInput = EditText(this@MainActivity).apply {
            //         inputType = InputType.TYPE_CLASS_NUMBER
            //         setEms(4)
            //         hint = "Priority"
            //         setText(this@MainActivity.getServicePriority(mimetype).toString())
            //         setOnFocusChangeListener { _, hasFocus ->
            //             if (!hasFocus) {
            //                 val value = text.toString().toIntOrNull()
            //                 if (value != null) this@MainActivity.setServicePriority(mimetype, value)
            //             }
            //         }
            //     }
            
            //     val row = LinearLayout(this@MainActivity).apply {
            //         orientation = LinearLayout.HORIZONTAL
            //         setPadding(0, 16, 0, 16)
            //         addView(checkbox, LinearLayout.LayoutParams(0, WRAP_CONTENT, 1f))
            //         addView(priorityInput, LinearLayout.LayoutParams(WRAP_CONTENT, WRAP_CONTENT))
            //     }
            
            //     binding.serviceConfigList.addView(row)
            // }
        }
    }

    private fun requestPermissions() {
        when {
            !hasGeneralPermissions() -> requestGeneralPermissions()
            !hasDrawOverlays() -> requestDrawOverlays()
            !hasCallRedirectionRole() -> requestCallRedirectionRole()
        }
    }

    private fun hasPermissions(): Boolean {
        return hasGeneralPermissions() && hasDrawOverlays() && hasCallRedirectionRole()
    }

    private fun requestDrawOverlays() {
        registerForDrawOverlays.launch(Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION))
    }

    private fun requestGeneralPermissions() {
        registerForGeneralPermissions.launch(PERMISSIONS)
    }

    private fun hasGeneralPermissions(): Boolean {
        return !PERMISSIONS.any { checkSelfPermission(it) != PackageManager.PERMISSION_GRANTED }
    }

    private fun hasDrawOverlays(): Boolean {
        return Settings.canDrawOverlays(this)
    }

    private fun requestCallRedirectionRole() {
        registerForCallRedirectionRole
            .launch(roleManager?.createRequestRoleIntent(RoleManager.ROLE_CALL_REDIRECTION))
    }

    private fun hasCallRedirectionRole(): Boolean {
        return roleManager?.isRoleHeld(RoleManager.ROLE_CALL_REDIRECTION) ?: false
    }
}
